/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "event.hpp"
#include "eventia.hpp"
#include "private/context.hpp"
#include <cassert>

namespace eve {

Event::Event (const Context& ctx) :
	m_eventia(ctx.eventia),
	m_was_stopped(false)
{
	assert(m_eventia);
	m_cb_id = m_eventia->register_stop_callback([this](){this->on_loop_stopping_local();});
}

Event::~Event() noexcept {
	m_eventia->unregister_stop_callback(m_cb_id);
	on_loop_stopping_local();
}

void Event::on_loop_stopping_local() noexcept {
	if (not m_was_stopped) {
		m_was_stopped = true;
		this->on_loop_stopping();
	}
}

void Event::set_exception (std::exception_ptr ptr) {
	m_eventia->set_exception(ptr);
}

} //namespace eve
