/* Copyright 2020, Michele Santullo
 * This file is part of orotool.
 *
 * Orotool is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Orotool is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Orotool.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "api_nap_exception.hpp"
#include "nap/http_response.hpp"
#include <string>

namespace oro {

ServerError::ServerError (const nap::HttpResponse& resp) :
	std::runtime_error(
		std::string("Server replied: ") + std::to_string(resp.code) + " (" +
			std::string(resp.http_ver) + ", \"" + std::string(resp.code_desc) +
			"\")"
	),
	m_err_code(static_cast<int>(resp.code))
{
}

int ServerError::error_code() const noexcept {
	return m_err_code;
}

} //namespace oro
